#!/bin/bash
dev=0
while getopts d option; do
	case "${option}" in
	d) dev=1; printf "Development flag set.\n"; sleep 2;;
	*) exit 1;;
	esac
done

TARGET=${TARGET:-ctdwa-774-cxnatest1}

#Get list of tests
declare -a tags
declare -a names
declare -a ptps


read_config() {

	while IFS=' ' read -r -a array; do
	    if [[ "${array[0]}" != "#"* ]] ; then 
		if [ "${#array[@]}" == "3" ] ; then
			tags+=(${array[0]})
			name=`echo ${array[1]}| tr '_-' ' ' `
			names+=("${name}")
			ptps+=(${array[2]})
		fi
	    fi
	done < tests.cfg
}

menu_header() {
	clear
	printf "#################################################\n"
	printf "# switch-release_tests / Target=$TARGET\n"
	printf "#################################################\n"
	printf "#"
}


run_command() {
	local cmd=$1
	echo $cmd	
	eval $cmd
}

wait_target_up() {
	until ping -c1 ${TARGET} >/dev/null 2>&1; do :; done
	echo ${TARGET} is responding to ping ... 
}

kill_ppsi() {
	local x=""
	while [ "$x" != "/wr/bin/ppsi" ]; do 
		sleep 5
	    echo "PPSi NOT yet running on target ...."
		x=`ssh root@${TARGET} ps ax | grep -o /wr/bin/ppsi 2>/dev/null`
	done
	echo "PPSi running on target ($x)"
	sleep 3
	run_command "ssh root@${TARGET} /etc/init.d/ppsi.sh stop"
}

run_test() {
	local idx=$1
	local tag=${tags[$idx]}
	local name=${names[$idx]}
	local ptp=${ptps[$idx]}
	local dirLocation="root@${TARGET}:/wr/etc/dot-config"
	echo "Running test: $tag $name"
	run_command "scp dot-config.$tag $dirLocation"
	run_command "ssh root@${TARGET} /sbin/reboot"
	sleep 5
	wait_target_up
	if [ "$ptp" == "n" ] ; then 
		kill_ppsi
	fi
	echo "Type enter to continue"
	IFS= read -r x
}

menu_main() {
	while : ; do
		menu_header
	    printf "Start test \n\n"
		for i in "${!tags[@]}"; do
		    ptpStr=""
		    if [ "${ptps[$i]}" == "n" ] ; then ptpStr="(PTP off)"; fi
		    printf "  %s) %s\n" "$i" "${tags[$i]} ${names[$i]} $ptpStr"
		done
		printf "  q) quit\n\n"
	        printf "  Choice : "
		IFS= read -r opt
		if [[ $opt =~ ^[0-9]+$ ]] && (( ($opt >= 0) && ($opt <= "${#tags[@]}") ));  then
	    		run_test "$opt"
		else
	   		if [ "$opt" == "q" ] ; then 
	    			exit 0;
	    	fi
	    fi
	done
}

read_config
menu_main
