#!/bin/bash

source genLib.sh

inputFile=$cleanLogName
fileHeader=`basename $inputFile .csv`

function latencyValueAdjustment() {
	echo `echo "scale=3;($1*1000)/1" | bc` # div by 1 to take into account the scale
}

function float3ValueAdjustment() {
	echo `echo "scale=3;$v/1" | bc` # div by 1 to take into account the scale
}

function portValueAdjustment() {
	echo `echo $1 | grep -o -e [0-9]$`
}

# Value adjusment functions
valueAdjustments[tkIdxLatencyAvg]=latencyValueAdjustment
valueAdjustments[tkIdxLatencyMin]=latencyValueAdjustment
valueAdjustments[tkIdxLatencyMax]=latencyValueAdjustment
valueAdjustments[tkIdxRxBer]=float3ValueAdjustment
valueAdjustments[tkIdxPort]=portValueAdjustment

function generateDataHeader() {
	local i=0
	
	echo -n "# "
	for idx in ${tkIdxs[@]} ; do
		echo -n " ${cvsTitles[$idx]} "
	done 
	echo ""
}

function generateData() {
	local line=$1
	local tokens
	local idx=0
	local i=0
	
	OLDIFS=$IFS
	IFS="," read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
		
	for tkIdx in ${tkIdxs[@]}; do 
		i=${csvColumn[$tkIdx]}
		v=${tokens[$i]}
		exp=`echo $v | grep -o -e E` 
		if [ -n "$exp" ] ; then # Convert notation
			v=`echo $v | awk '{printf("%.10f\n", $1)}'`
		fi
		if [ -n "${valueAdjustments[tkIdx]}" ] ; then	
			v=$( ${valueAdjustments[tkIdx]} $v ) 
		fi
		if [[ "$v" = [.]* ]] ; then v="0$v"; fi # All value strating with .xxx changed to 0.xxx  
		if [[ "$v" = [-]* ]] ; then >&2 echo "$v $tkIdx  ${tokens[$i]}"; fi 
 		echo -n "$v "
 	done
 	echo ""
}

function learningFromCsv () {
	local line=$1
	local tokens
	
	
	
	OLDIFS=$IFS
	IFS="," read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
	
	for idx in "${!tokens[@]}"; do
		local j=0
		token=${tokens[$idx]}
		
		for j in  "${!cvsTitles[@]}"; do
	    	if [[ "$token" = ${cvsTitles[$j]}* ]] ; then 
	    	 	csvColumn[$j]=$idx
	    	 	break;
	    	fi
	    done
	done
	displayDebugInfo
}

function generateFilterNoErrorsData() {
	local line="$1"
	local list=( $tkIdxRxFcsErrors $tkIdxRxSeqErr $tkIdxRxMisErr $tkIdxRxPldErr $tkIdxRxBer	)
	
	OLDIFS=$IFS
	IFS=" " read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
	
	for tkIdx in ${list[@]}; do  
		if [ "${tokens[ ${plotDataIdx[$tkIdx]} ]}" -ne "0" ] ; then
			line=""
			break
		fi
	done
	echo "$line"
}

lastLine=( "" "" "" "" )
lastFrameSize=( 0 0 0 0 )
dataFound=( 0 0 0 0 )
generateLimitsDataRetValue="" # like this because we modify global variable in the function

function generateLimitsData() {
	local line="$1"
	local list=( tkIdxRxFcsErrors tkIdxRxSeqErr tkIdxRxMisErr tkIdxRxPldErr tkIdxRxBer	)
	local str=""
	
	if [ -n "$line" ] ; then 
		OLDIFS=$IFS
		IFS=" " read -a tokens <<< "$( echo $line )"
		IFS=$OLDIFS
		
		port=${tokens[ ${plotDataIdx[$tkIdxPort]}]}
		frameSize=${tokens[ ${plotDataIdx[$tkIdxFrameLength]} ]}
		if [ "$frameSize" -ne "${lastFrameSize[$port]}" ] ; then 
			# new size: 
			if [ "${dataFound[$port]}" -eq "0" ] ; then 
				str="${lastLine[$port]}"
			fi
			lastFrameSize[$port]=$frameSize
			dataFound[$port]=0 
		else
			# Same frame size
			if [ "${dataFound[$port]}" -eq "0" ] ; then
				# not found yet 
				for tkIdx in ${list[@]} ; do  
					if [ "${tokens[ ${plotDataIdx[$tkIdx]} ]}" -ne "0" ] ; then
						# Error detected
						str="${lastLine[$port]}"
						lastFrameSize[$port]=$frameSize
						dataFound[$port]=1
						break
					fi
				done
			fi
		fi
		lastLine[$port]="$line"
	else
		# empty ligne is to trigger the eof
		str=""
		for port in ${!dataFound[@]}; do
			if [ "${dataFound[$port]}" -eq "0" ] ; then
				if [ -n "$str" ] ; then  str+=$'\n'; fi
				str+="${lastLine[$port]}"
			fi
		done
	fi
	generateLimitsDataRetValue="$str"
}

function genLimitsDataFile() {
	>&2 echo " Generate limits data file \"$dataFileLimits\""
	local n=1
	local lineCnt=`wc -l  $inputFile | grep -o ^[0-9]*`
	while read line  ; do
		if [ "$n" -eq "1" ] ; then  
			echo "$line" >$dataFileLimits
		else 
			generateLimitsData "$line"
			if [ -n "$generateLimitsDataRetValue" ] ; then echo "$generateLimitsDataRetValue" >>$dataFileLimits; fi
		fi 
		n=$((n+1))
		displayPercentage $lineCnt $n
	done < $dataFile
	generateLimitsData "" # Inform EOF
	if [ -n "$generateLimitsDataRetValue" ] ; then echo "$generateLimitsDataRetValue" >>$dataFileLimits; fi
	clearDisplayPercentage
	# Split in different files
	for p in `seq 0 $(( $portCount-1 ))`; do
		of=`basename $dataFileLimits .dat`_$p.dat
		>&2 echo "Generating $of "
		grep -e "^#" -e "^$p" $dataFileLimits >$of
	done
}

function genFilteredNoErrorsDataFile() {
	>&2 echo " Generate filtered data file \"$dataFileFilteredNoErrors\""
	local n=1
	local lineCnt=`wc -l  $inputFile | grep -o ^[0-9]*`
	while read line  ; do
		if [ "$n" -eq "1" ] ; then  
			echo "$line" >$dataFileFilteredNoErrors
		else 
			local str=$( generateFilterNoErrorsData "$line" )
			if [ -n "$str" ] ; then echo "$str" >>$dataFileFilteredNoErrors; fi
		fi 
		n=$((n+1))
		displayPercentage $lineCnt $n
	done < $dataFile
	clearDisplayPercentage
}

function genDataFile () {
	>&2 echo "Reading CSV file \"$inputFile\" and generate data file \"$dataFile\""
	local n=1
	local lineCnt=`wc -l  $inputFile | grep -o ^[0-9]*`
	while read line  ; do
		if [ "$n" -eq "1" ] ; then  
			learningFromCsv "$line"
			generateDataHeader >$dataFile
		else 
			generateData "$line" >>$dataFile
		fi 
		n=$((n+1))
		displayPercentage $lineCnt $n
	done < $inputFile
	clearDisplayPercentage
} 

function main () {
	genDataFile
	scanData
	#portCount=4
	#plotDataIdx=( [0]=0 [1]=1 [2]=2 [3]=3 [4]=4 [5]=5 [6]=6 [7]=7 [8]=8 [9]=9 [10]=10 )
	displayDebugInfo
	genFilteredNoErrorsDataFile
	genLimitsDataFile
}

main

