#!/bin/bash

source genLib.sh

function generatePlotScript() {
	local title=$1
	local outFile=$2
	local using=$3
	local dataF=$4
	
	echo "set   autoscale"
	echo "unset log"
	echo "unset label"
	echo "set xtics auto"
	echo "set ytic auto"
	echo "set ztic auto"
	echo "set pointsize 0.5"
	echo "set title \"$title\""
	echo "set xlabel \"frame Length (Bytes)\""
	echo "set ylabel \"NetLoad (%)\""
	echo "unset zlabel"
	echo "set dgrid3d 30,30"
	echo "set hidden3"
	echo "set style data lines"
	echo "set pm3d "
	echo "set view 68,332"
	echo "set key noautotitle"
	echo "set terminal png size 1024,900"      
	echo "set output '$outFile.png'"
	echo "splot \"$dataF\" using $using with lines"
}

function generatePlotScriptLimits() {
	local title=$1
	local outFile=$2
	local plotCommand=$3
	
	echo "set   autoscale"
	echo "unset log"
	echo "unset label"
	echo "set xtics auto"
	echo "set ytic auto"
	echo "set pointsize 0.5"
	echo "set title \"$title\""
	echo "set xlabel \"frame Length (Bytes)\""
	echo "set ylabel \"NetLoad (%)\""
	echo "set style data lines"
	echo "set key noautotitle"
	echo "set terminal png size 1024,900"      
	echo "set output '$outFile.png'"
	echo "$plotCommand"
}

declare -a plotFiles

function generateImages() {
	rm -f ../*.png
	for idx in ${!plotFiles[@]}; do
		f=${plotFiles[$idx]}
		echo " Generating image for $f"
		gnuplot $f
	done
}

function generatePlotScriptsLimits() {

	pf="limits"
	of="$pf.p"
	plotCommand="plot"
	for p in `seq 0 $(( $portCount-1 ))`; do
		t="Port $p"
		df=`basename $dataFileLimits .dat`_$p.dat
		if [ "$p" -ne "0" ] ; then plotCommand+=", "; fi
		plotCommand+=" \"$df\" using 2:3 title \"$t\""
	done
	generatePlotScriptLimits "Functioning limits" ../$pf "$plotCommand"  >$of
	plotFiles+=( $of )

}

function generatePlotScriptsLatencies() {
	local titles=( "Average Latency (us)" "Min Latency (us)" "Max Latency (us)"
		"Average Latency - No Errors (us)" "Min Latency - No Errors (us)" "Max Latency - No Errors (us)")
	local pngFiles=( latencyAvg latencyMin latencyMax 
		latencyAvgNoErr latencyMinNoErr latencyMaxNoErr )
	local tkIdxList=( $tkIdxLatencyAvg $tkIdxLatencyMin $tkIdxLatencyMax
		$tkIdxLatencyAvg $tkIdxLatencyMin $tkIdxLatencyMax )
	local dataFiles=( $dataFile $dataFile $dataFile 
		$dataFileFilteredNoErrors $dataFileFilteredNoErrors $dataFileFilteredNoErrors)
	
	for  i in ${!titles[@]}; do 
		for p in `seq 0 $(( $portCount-1 ))`; do
			# Latencies
			pf="${pngFiles[$i]}_$p"
			of="$pf.p"
			t="${titles[$i]} - Port $p"
			tkIdx=${tkIdxList[$i]}
			z=$(( ${plotDataIdx[$tkIdx]}+1 )) # +1 because GNUPLOT indexes start at 1
			u="2:3:(\$1==$p ? \$$z : 1/0)"
			generatePlotScript "$t" ../$pf "$u" ${dataFiles[$i]} >$of
			plotFiles+=( $of )
		done
	done

}

function generatePlotScriptsErrors() {
	local titles=( "RxFcsErrors" "RxSeqErr" "RxMisErr" "RxPldErr" "RxBer" )
	local pngFiles=( rxFcsErrors rxSeqErr rxMisErr rxPldErr rxBer )
	local tkIdxList=( $tkIdxRxFcsErrors $tkIdxRxSeqErr $tkIdxRxMisErr $tkIdxRxPldErr $tkIdxRxBer )
	local dataFiles=( $dataFile $dataFile $dataFile $dataFile $dataFile )
	
	
	for  i in ${!titles[@]}; do 
		tkIdx=${tkIdxList[$i]}
		# Latencies
		pf="${pngFiles[$i]}"
		of="$pf.p"
		z=$(( ${plotDataIdx[$tkIdx]}+1 ))  # +1 because GNUPLOT indexes start at 1
		generatePlotScript "${titles[$i]}" ../$pf 2:3:$z ${dataFiles[$i]} >$of
		if [ "${valuesMax[$tkIdx]}" != "0" ] ; then 
			plotFiles+=( $of );
			echo "File $of will be plotted (valueMax= ${valuesMax[$tkIdx]})"
		fi
	done
	
	
}

function generatePlotScripts() {
	rm -f *.p
	generatePlotScriptsLatencies
	generatePlotScriptsErrors
	generatePlotScriptsLimits
}


function main () {
	>&2 echo "Generate 3D plots"
	#plotDataIdx=( [0]=0 [1]=1 [2]=2 [3]=3 [4]=4 [5]=5 [6]=6 [7]=7 [8]=8 [9]=9 [10]=10 )
	#portCount=4
	scanData
	>&2 echo "Generate plot  ..."
	generatePlotScripts 	
	>&2 echo "Generate Images ..."
	generateImages
} 


main
