
dataFile=plot.dat
dataFileFilteredNoErrors=plot_no_errors.dat
dataFileLimits=plot_limits.dat
dataFileTransposed=plot_trp.dat
dbg="1"

function enum() {
	local -a list=("${!1}")
	local i
	local x=0
	for i in ${list[@]}; do 
	  readonly ${i}=$((x++)); done
}

function list() {
	local name=$1
	local -a list=("${!2}")

	for i in ${!list[@]}; do 
		eval "${name}[i]=${list[$i]}"
	done
}

configFile=config.txt
logName=""
cleanLogName=""
dataCollectionName=""

function decodeConfig() {
	local line=""
	
	while read line  ; do
		if [[ $line = [a-zA-Z]*  ]] ; then 
	
			OLDIFS=$IFS
			IFS="=" read -a tokens <<< "$( echo $line )"
			IFS=$OLDIFS
			
			case ${tokens[0]} in
				"logName" ) logName=${tokens[1]};;
				"cleanLogName" ) cleanLogName=${tokens[1]};;
				"dataCollectionName" ) dataCollectionName="${tokens[1]}";;
			esac
		fi
	done < $configFile
}

function genInit() {
	#Set default max values
	for tkIdx in ${tkIdxs[@]} ; do
		valuesMax[$tkIdx]=${valuesMax[$tkIdx]--9999999999}
	done	
	decodeConfig
} 

# Values max indexed by tkIdx... 
declare -a valuesMax

# Column in Csv file indexed by tkIdx... 
declare -a csvColumn

# Number of ports found
portCount=${portCount--1} # If exist do not overwrite it

# Token idx names
tkIdxNames=(
tkIdxPort
tkIdxFrameLength
tkIdxNetLoad
tkIdxRxFcsErrors
tkIdxRxSeqErr
tkIdxRxMisErr
tkIdxRxPldErr
tkIdxRxBer
tkIdxLatencyAvg
tkIdxLatencyMin
tkIdxLatencyMax
)

# Create enumeration and list
enum tkIdxNames[@]
list tkIdxs tkIdxNames[@]
genInit 

declare  -a cvsTitles=(
[$tkIdxFrameLength]="FrameLength"
[$tkIdxNetLoad]="NetLoad"
[$tkIdxRxFcsErrors]="RxFcsErrors"
[tkIdxRxSeqErr]="RxSeqErr"
[tkIdxRxMisErr]="RxMisErr"
[tkIdxRxPldErr]="RxPldErr"
[tkIdxRxBer]="RxBer"
[tkIdxLatencyAvg]="LatencyAvg"
[tkIdxLatencyMin]="LatencyMin"
[tkIdxLatencyMax]="LatencyMax"
[tkIdxPort]="SrcPort"
)

function displayPercentage() {
	local maxValue=$1
	local currValue=$2

	modulo=`echo "scale=0; $currValue%20" | bc`
	if [ "${modulo}" == "0" ] ; then 
	   p=`echo "scale=0; ($currValue*100)/$maxValue" | bc`
	   echo -ne "\r $p%"
	fi
} >&2

function clearDisplayPercentage() {
	echo -ne "\r     \r" 
} >&2


# Gnluplot data index indexed by tkIdx.....
declare -a plotDataIdx

function printArray() {
	local arr=("${!1}")
	local ret=""
	local i=0
	
	for i in ${!arr[@]} ; do
		ret+="[$i]=${arr[$i]} "
	done 
	echo -n $ret
}

function displayDebugInfo() {
	if [ -n "$dbg" ] ; then
		echo "portCount=$portCount"
		echo -n "tkIdxs= "; echo $( printArray tkIdxs[@]] )
		echo -n "tkIdxNames: "; echo $( printArray tkIdxNames[@]] )
		echo -n "cvsTitles: "; echo $( printArray cvsTitles[@]] )
		echo -n "plotDataIdx: "; echo $( printArray plotDataIdx[@]] )
		echo -n "csvColumn: "; echo $( printArray csvColumn[@]] )
		echo -n "valuesMax: "; echo $( printArray valuesMax[@]] )
		echo -n "logName: "; echo $logName
		echo -n "cleanLogName: "; echo $cleanLogName
		echo -n "dataCollectionName: "; echo $dataCollectionName
	fi	
} >&2 

function scanData() {
	local line=""
	local n=0
	
	>&2 echo Scanning  $dataFile ...
	if [ "$portCount" -eq "-1" ] ; then 
		local lineCnt=`wc -l  $dataFile | grep -o ^[0-9]*`
		portCount=0
		while read line ; do
			if [[ $line =~ ^#.* ]] ; then
				getDataPosition "$line"  
			else 
				getDataOverview  "$line"
			fi 
			n=$(( $n+1 ))
			displayPercentage $lineCnt $n
		done < $dataFile
		# Update port count
		portCount=$(( ${valuesMax[$tkIdxPort]}+1 ))
	clearDisplayPercentage
	fi
	displayDebugInfo
}


function getDataPosition () {
	local line="$1"
	
	OLDIFS=$IFS
	IFS=" " read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS

	local token=0
	local i=0
	
	for token in ${tokens[@]} ; do
		if [[ $token =~ ^[A-Za-z].* ]] ; then
			local j=0   
			for j in ${!cvsTitles[@]} ; do
				local label=${cvsTitles[$j]}
				if [ "$label" == "$token" ] ; then
					 plotDataIdx[$j]=$i
				fi 
			done
			i=$(( $i+1 ))
		fi
	done 
}

function getDataOverview() {
	local line="$1"

	if [[ $line = [0-9-+]* ]] ; then 

		OLDIFS=$IFS
		IFS=" " read -a tokens <<< "$( echo $line )"
		IFS=$OLDIFS

		for tkIdx in ${tkIdxs[@]} ; do 
	  		local idx=${plotDataIdx[$tkIdx]} 
			local value=${tokens[$idx]}
			if (( $(echo "$value > ${valuesMax[$tkIdx]}" |bc -l) )) ; then valuesMax[$tkIdx]=$value; fi
		done
	fi
}

