#!/bin/bash

source genLib.sh

tmpFile1=${cleanLogName}_1.tmp
tmpFile2=${cleanLogName}_2.tmp
#tmpFile=/dev/stdout

# Tx frame tag
txFrameMax=4200000

lastTxFrames=( 0 0 0 0  )
streamLines=( "" "" "" "" )
streamSynchronized=( 0 0 0 0 )

#list of columns
tkStream=1
tkTxFrames=6

function learning () {
	local line=$1
	local tokens
	
	OLDIFS=$IFS
	IFS="," read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
	for i in "${!tokens[@]}"; do
	    if [ "${tokens[$i]}" == "TxFrames" ] ; then 
	    	 tkTxFrames=$i
	    	 break
	    fi
	done
}

function formatHeader() {
	local line=$1
	
	s=${line/Timestamp/'Timestamp,FrameLength,NetLoad'}
	echo $s
}

function formatData() {
	local line=$1
	
	OLDIFS=$IFS
	IFS=",\n" read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
	
	local txFrames=${tokens[$tkTxFrames]}
	local netLoad=`echo "scale=0; $txFrames%100" | bc`
	local subFrameLength=0
	
	if [ "$netLoad" -eq "0" ] ; then 
		subFrameLength=1
		netLoad=100 
	fi
		
	local frameLength=`echo "scale=0; ($txFrames%1000000)/100-$subFrameLength" | bc`
	local ret="${tokens[0]},"	
	ret+="$frameLength,"	
	ret+="$netLoad,"	
	local i=0
	for value in ${tokens[@]:1}; do
		if [ "$i" -gt "0" ] ; then ret+=","; fi
		ret+="$value"
		i=$(( $i+1 ))
	done
	echo "$ret"
}

function getStreamIndex() {
	local stream=$1
	echo `echo $stream | grep -o -e "[0-9]*$"`
}

function lineDecoder () {
	local line=$1
	local tokens
	
	OLDIFS=$IFS
	IFS="," read -a tokens <<< "$( echo $line )"
	IFS=$OLDIFS
	local streamId=$( getStreamIndex  ${tokens[$tkStream]} )
	local txFrames=${tokens[$tkTxFrames]}

	if 	[  -z "$txFrames" ] ; then return 0 ; fi
	if  [ "$txFrames" -gt "${txFrameMax}" ] ; then return 0; fi

	if 	[ "$txFrames" -lt "${lastTxFrames[$streamId]}" ] ; then
			formatData "${streamLines[$streamId]}"
	fi
	lastTxFrames[$streamId]=$txFrames
	streamLines[$streamId]="$line"
}

## main
function main() {
	local n=1
	local lineCnt=`wc -l  $logName | grep -o ^[0-9]*`
	
	sed -i "1s/,Port,/,SrcPort,/" $logName # Change Port in the header by SrcPort
	
	while read line ; do
		if [ "$n" -eq "1" ] ; then  
			learning $line
			formatHeader "$line" 
		else if [ "$n" -gt "2" ] ; then
				lineDecoder "$line"
			fi
		fi 
		n=$((n+1))
		#if [ "$n" -gt "10000" ] ; then break; fi
	    displayPercentage $lineCnt $n
	done < $logName
	clearDisplayPercentage
} 

main > $tmpFile1
cat $tmpFile1 | sed -e "s/\r//g" | sed 's/[,]$//' > $tmpFile2
(head -n 1 $tmpFile2 && tail -n +2  $tmpFile2 | sort -t  $',' -k 1,1 -k 2,2 -k 3,3 -k 4,4) >$cleanLogName
rm -f *.tmp
