#!/bin/sh

source schedLib.sh

#
# Options
#
STREAMS_PER_PORT=1 # default
LEARNING_PROCESS=1 # default

while getopts "s:l" arg; do
  case $arg in
    s)
      STREAMS_PER_PORT=$OPTARG
      ;;
    l)
      LEARNING_PROCESS=0
      ;;
  esac
done

SCHED_NAME="sched_fp_incr_63_s$STREAMS_PER_PORT"
if [ "$LEARNING_PROCESS" -eq "0" ] ; then
	SCHED_NAME+="_nl"
fi

OUTPUT=$SCHED_NAME.xsch

rates=(       10 30   50  70 80 85 90 92 95 97 99 100 )
timeLowSec=(  40 20   15  15 15 15 15 15 15 15 15  15 )  
timeHighSec=(550 300 120 105 90 75 70 70 70 70 70  70 )
minPacketSize=64
maxPacketSize=1522
packetSizeIncrement=63
packetSizes=`seq $minPacketSize $packetSizeIncrement $maxPacketSize`
packetSizes+=" 1522"
packetLimit=`echo "scale=0; 4000000/${STREAMS_PER_PORT}/1000000*1000000" | bc`
#packetLimit=10


#
# Start generation of the scheduler
#
schedHeader $SCHED_NAME

schedPortStopTraffic true ""
schedPortClearStats true ""
schedStreamEnableStream true ""

# Learning process
if [ "$LEARNING_PROCESS" -eq "1" ] ; then
	schedStreamCustom true "" "PS_PACKETLIMIT 10"
	schedStreamCustom true "" "PS_PACKETLENGTH FIXED 64 64"
	schedStreamSetParameter true "" PS_RATEFRACTION 50.0
	schedPortStartTraffic true ""
	schedWait 2.0 
	schedPortStopTraffic true ""
fi
	
# Synchronize loggoing 
schedPortClearStats true ""
schedWait 20.0 

#
for packetSize in $packetSizes; do
	schedStreamCustom true "" "PS_PACKETLENGTH FIXED $packetSize $packetSize"
	for i in "${!rates[@]}"; do
		rate=${rates[$i]}
		ratePerPort=`echo "scale=2; $rate/$STREAMS_PER_PORT" | bc`
		schedStreamSetParameter true "" PS_RATEFRACTION $ratePerPort
		pckCount=`echo "scale=0;$packetLimit+($packetSize*100)+$rate" | bc`
		schedStreamCustom true "" "PS_PACKETLIMIT $pckCount"
		schedPortClearStats true ""
		# Wait a little bit for logs
		schedWait 6.0 
		schedPortStartTraffic true ""
		# calculate duration 
	    a=`echo "scale=4;(${timeHighSec[$i]}-${timeLowSec[$i]})/($maxPacketSize-$minPacketSize)" | bc`
		b=`echo "scale=4;${timeHighSec[$i]}-($a*$maxPacketSize)" | bc`
		y=`echo "scale=0;$a*$packetSize+$b" | bc`
		schedWait $y
		schedPortStopTraffic true ""		
	done
done
schedFooter
