#!/bin/sh

source schedLib.sh

#
# Options
#
STREAMS_PER_PORT=1 # default

while getopts "s:" arg; do
  case $arg in
    s)
      STREAMS_PER_PORT=$OPTARG
      ;;
  esac
done

SCHED_NAME="sched_fp_incr_fixed_imb_s$STREAMS_PER_PORT"
OUTPUT=$SCHED_NAME.xsch

rates=( 50  80  95   100 )
packetSizes=( 64 65 700 701 1521 1522 ) 
#rates=( 30 40 50 )
#packetSizes=( 64 1522 ) 
bursTime=300.0
#bursTime=10.0

mainStreams="S-0-0-0-0,S-0-0-1-0"
mainPorts="P-0-0-0-0,P-0-0-0-1"
secondaryStreams="S-0-0-2-0,S-0-0-3-0"
secondaryPorts="P-0-0-0-2,P-0-0-0-3"

function waitTime() {
	local duration=$1
	schedWait $duration
}

function genLoopInternal() { 
	local ports=$1
	local streams=$2
	local packetSize=0
	
	for packetSize in ${packetSizes[@]}; do
		schedStreamCustom false $streams "PS_PACKETLENGTH FIXED $packetSize $packetSize"
		for i in "${!rates[@]}"; do
			local rate=${rates[$i]}
			local ratePerPort=`echo "scale=2; $rate/$STREAMS_PER_PORT" | bc`
			schedStreamSetParameter false $streams PS_RATEFRACTION $ratePerPort
			schedPortClearStats true ""
			# Wait a little bit for logs
			schedWait 10.0
			schedPortStartTraffic true ""
			waitTime $bursTime
			schedPortStopTraffic true ""	
		done
	done
}


function genLoopMain() { 
	local ports=$1
	local streams=$2
	local packetSize=0
	local i=0
	
	for packetSize in ${packetSizes[@]}; do
		schedStreamCustom false $streams "PS_PACKETLENGTH FIXED $packetSize $packetSize"
		for i in "${!rates[@]}"; do
			local rate=${rates[$i]}
			local ratePerPort=`echo "scale=2; $rate/$STREAMS_PER_PORT" | bc`
			schedStreamSetParameter false $streams PS_RATEFRACTION $ratePerPort
			genLoopInternal  $secondaryPorts $secondaryStreams 
		done
	done
}

schedHeader $SCHED_NAME

# Learning process
schedPortStopTraffic true ""
schedStreamEnableStream true ""
schedStreamCustom true "" "PS_PACKETLIMIT 100"
schedStreamCustom true "" "PS_PACKETLENGTH FIXED 64 64"
schedStreamSetParameter true "" PS_RATEFRACTION 50.0
schedPortStartTraffic true ""
schedWait 2.0 
schedPortStopTraffic true ""

# Synchronize logging 
schedPortClearStats true ""
schedWait 20.0 


schedStreamCustom true "" "PS_PACKETLIMIT 0" # Unlimited duration

genLoopMain  $mainPorts $mainStreams 
schedFooter
